#include "shader.h"

shader_t * shaderCompile(char *vertexShaderSource, char* fragmentShaderSource) {
  int isSuccess, maxLength;
  char *error;
  GLuint shaderVertex, shaderFragment, shaderProgram;
  
  
  // Load the vertex shader first
  shaderVertex = glCreateShader(GL_VERTEX_SHADER);
  glShaderSource(shaderVertex, 1, &vertexShaderSource, 0);
  glCompileShader(shaderVertex);

  // Validate
  glGetShaderiv(shaderVertex, GL_COMPILE_STATUS, &isSuccess);
  if(!isSuccess) {
    glGetShaderiv(shaderVertex, GL_INFO_LOG_LENGTH, &maxLength);
    error = malloc(maxLength);
    glGetShaderInfoLog(shaderVertex, maxLength, &maxLength, error);
    printf("Failed to compile vertex shader %s\n", error);
    free(error);
    return NULL;
  }

  // Now load the Frag shader
  shaderFragment = glCreateShader(GL_FRAGMENT_SHADER);
  glShaderSource(shaderFragment, 1, &fragmentShaderSource, 0);
  glCompileShader(shaderFragment);
  glGetShaderiv(shaderFragment, GL_COMPILE_STATUS, &isSuccess);
  if(!isSuccess) {
    glGetShaderiv(shaderFragment, GL_INFO_LOG_LENGTH, &maxLength);
    error = malloc(maxLength);
    glGetShaderInfoLog(shaderFragment, maxLength, &maxLength, error);
    printf("Failed to compile fragment shader %s\n", error);
    free(error);
    glDeleteShader(shaderVertex);
    return NULL;
  }

  // Now create the shader program.
  shaderProgram = glCreateProgram();
  glAttachShader(shaderProgram, shaderVertex);
  glAttachShader(shaderProgram, shaderFragment);

  //Bind, Verify & Use the shader program
  glLinkProgram(shaderProgram);
  glGetProgramiv(shaderProgram, GL_LINK_STATUS, &isSuccess);
  if(!isSuccess) {
    glGetProgramiv(shaderProgram, GL_INFO_LOG_LENGTH, &maxLength);
    error = malloc(maxLength);
    glGetProgramInfoLog(shaderProgram, maxLength, &maxLength, error);
    printf("Failed to load shader program %s\n", error);
    free(error);
    glDeleteShader(shaderVertex);
    glDeleteShader(shaderFragment);
    return NULL;
  }

  // Everything is okay, let's create the encapsulated shader.
  shader_t *shader = malloc(sizeof(shader_t));
  if(shader == NULL) {
    glDeleteProgram(shaderProgram);
    glDeleteShader(shaderVertex);
    glDeleteShader(shaderFragment);
    return NULL;
  }
  shader->shaderVertex = shaderVertex;
  shader->shaderFrag = shaderFragment;
  shader->shaderProgram = shaderProgram;

  // Bind the shader
  shaderUse(shader);

  // Fetch the uniforms.
  return shader;
}

bool shaderDipose(shader_t *shader) {
  glDeleteProgram(shader->shaderProgram);
  glDeleteShader(shader->shaderVertex);
  glDeleteShader(shader->shaderFrag);
  free(shader);
  return true;
}

void shaderUse(shader_t *shader) {
  glUseProgram(shader->shaderProgram);
}